/***************************************************************************
                          icebgset.cpp  -  description
                             -------------------
    begin                :    3 00:47:59 EEST 2002
    copyright            : (C) 2002 by Vadim A. Khohlov
    email                : xvadim@teko.kherson.ua
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>

#include <qapp.h>

#include <qlayout.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qgroupbox.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qmessagebox.h>
#include <qtooltip.h>

#include <qcolor.h>

#include <qcolordialog.h>
#include <qfiledialog.h>

#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qstring.h>

#include "icebgset.h"

QString DesktopBgColorStr("DesktopBackgroundColor");
QString DesktopBgImageStr("DesktopBackgroundImage");
QString DesktopBgCenteredStr("DesktopBackgroundCenter");

IceBgSetApp::IceBgSetApp():
icewmCfgDir(getenv("ICEWM_PRIVCFG"))
{
    if(icewmCfgDir.isNull())
        icewmCfgDir = QDir::homeDirPath() + "/.icewm";
        
    QFile prefFile(icewmCfgDir + "/preferences");
	if(!prefFile.exists())
	{
		QMessageBox::critical(0, tr("Oops!"),tr("Can't load preferences file"));
		qApp->quit();
	} /* 	if(!prefFile.exest()) */	
    setCaption(tr("IceBgSet " VERSION));

    QVBoxLayout *topLayout = new QVBoxLayout(this, 0);
    CHECK_PTR(topLayout);

    /* Color's section */
    QGroupBox *gbColor = new QGroupBox(2, Horizontal, tr("&Color:"), this);
    CHECK_PTR(gbColor);
    topLayout->addWidget(gbColor);
    edColor = new QLineEdit(gbColor);
    CHECK_PTR(edColor);
    QPushButton *pbColor = new QPushButton(tr("C&hange..."), gbColor);
    CHECK_PTR(pbColor);
    connect(pbColor, SIGNAL(clicked()), SLOT(slotChangeColor()));

    /* Image's section */
    QGroupBox *gbImage = new QGroupBox(2, Horizontal, tr("Image:"), this);
    CHECK_PTR(gbImage);
    topLayout->addWidget(gbImage);

    QVBox *vbFiles = new QVBox(gbImage);
    CHECK_PTR(vbFiles);
    QLabel *lImages = new QLabel(tr("&Images:"), vbFiles);
    CHECK_PTR(lImages);
    lbImages = new QListBox(vbFiles);
    CHECK_PTR(lbImages);
    connect(lbImages, SIGNAL(selectionChanged(QListBoxItem*)),
	                  SLOT(slotImageChanged(QListBoxItem*)));
    lImages->setBuddy(lbImages);
    QPushButton *pbReload = new QPushButton(tr("&Reload images"), vbFiles);
    connect(pbReload, SIGNAL(clicked()), SLOT(slotReloadImages()));

    QVBox *vblPreview = new QVBox(gbImage);
    CHECK_PTR(vblPreview);
    QGroupBox *gbDir = new QGroupBox(2, Horizontal, tr("&Directory:"), vblPreview);
    CHECK_PTR(gbDir);
    edDirectory = new QLineEdit(gbDir);
    CHECK_PTR(edDirectory);
    QPushButton *pbDir = new QPushButton(tr("&Select..."), gbDir);
    CHECK_PTR(pbDir);
    connect(pbDir, SIGNAL(clicked()), SLOT(slotDirSelect()));
    cScaleImage = new QCheckBox(tr("Sc&ale image"), vblPreview);
    CHECK_PTR(cScaleImage);
    cCenteredBg = new QCheckBox(tr("D&esktop background centered and not tiled"), vblPreview);
    CHECK_PTR(cCenteredBg);

    lPreview = new QLabel(vblPreview);
    CHECK_PTR(lPreview);
    lPreview->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    lPreview->setMinimumSize(320, 200);
    lPreview->setAlignment(AlignCenter | ExpandTabs);
	vblPreview->setStretchFactor(lPreview, 10);

	cbIceWMVersion = new QCheckBox(tr("&IceWM's version < 1.2.10"), vblPreview);
	CHECK_PTR(cbIceWMVersion);
	QToolTip::add(cbIceWMVersion, tr("If this control is checked the icewm will "
				"be restarted.\nOtherwise the icewmbg will be restarted."));

    /* Buttons' section */
    QHBox *hbButtons = new QHBox(this);
    CHECK_PTR(hbButtons);
    topLayout->addWidget(hbButtons);
    QPushButton *pbOk = new QPushButton("&Ok", hbButtons);
    QPushButton *pbApply = new QPushButton(tr("&Apply"), hbButtons);
    QPushButton *pbClose = new QPushButton(tr("C&lose"), hbButtons);

    connect(pbClose, SIGNAL(clicked()), qApp, SLOT(quit()));
    connect(pbOk,    SIGNAL(clicked()),       SLOT(slotOk()));
    connect(pbApply, SIGNAL(clicked()),       SLOT(slotSetOptions()));

    loadCurrentSettings();

}//constructor

IceBgSetApp::~IceBgSetApp()
{
}

/** 
 * Opens colordialog for changing background color
 */
void IceBgSetApp::slotChangeColor()
{
    int r, g, b;
    r = g = b = 0;
    sscanf(edColor->text().data(), "rgb:%x/%x/%x", &r, &g, &b);
    QColor newBgColor = QColorDialog::getColor(QColor(r, g, b), this);

    if(newBgColor.isValid())
    {
        char stringColor[13];
        sprintf(stringColor, "rgb:%02x/%02x/%02x", newBgColor.red(),
                                                newBgColor.green(),
                                                newBgColor.blue());
        edColor->setText(stringColor);
        lPreview->setBackgroundColor(newBgColor);
    }/* if(newBgColor.isValid()) */
}/* void IceBgSetApp::slotChangeColor() */

/**
 * Reloads list of images
 */
void IceBgSetApp::slotReloadImages()
{
    QDir dir(edDirectory->text());

    lbImages->clear();
    lbImages->insertItem(tr("<none>"));

    dir.setNameFilter("*.png *.xpm *.jpg *.gif *.bmp *.img");
    const QFileInfoList *list = dir.entryInfoList(QDir::Files);
    if(!list)
    {
		QMessageBox::warning(this, tr("Error"), tr("Can't read ") + dir.absPath());
		return;
    }/* if (!list) */
    QFileInfoListIterator it( *list );      // create list iterator
    QFileInfo *fi;                          // pointer for traversing

    /* Scaning directory*/
    while ( (fi=it.current()) )
    {
        ++it;
        if(fi->fileName()[0] == '.')
            continue;
        lbImages->insertItem(fi->fileName());
    }/* while ( (fi=it.current()) ) */

}/* IceBgSetApp::slotReloadImage() */

/** 
 * Opens dialog for choosing directory with images
 */
void IceBgSetApp::slotDirSelect()
{
    QString dirName(edDirectory->text());
    dirName = QFileDialog::getExistingDirectory(dirName, this, "dirDialog",
              tr("Select directory with images"), true);
    if(!dirName.isEmpty())
    {
        edDirectory->setText(dirName);
        slotReloadImages();
    }/* if(!dirName.isEmpty()) */
}/* IceBgSetApp::slotDirSelect() */

/** No descriptions */
void IceBgSetApp::slotImageChanged(QListBoxItem *item)
{
    if(lbImages->currentItem() < 0)
        return;
    if(!lbImages->currentItem())
    {
        lPreview->setText(tr("no Image"));
        return;
    }/* if(!lbImages->currentItem()) */
    QString imageFile(edDirectory->text());
    imageFile += item->text();

    setPreviewImage(imageFile);
}/* IceBgSetApp::slotImageChanged(QListBoxItem *item) */

/** No descriptions */
void IceBgSetApp::slotOk()
{
    slotSetOptions();
    qApp->quit();
}/* IceBgSetApp::slotOk() */

/** No descriptions */
void IceBgSetApp::slotSetOptions()
{
    QString bgColor(DesktopBgColorStr);
    QString bgImageStr(DesktopBgImageStr);
    QString bgCentered(DesktopBgCenteredStr);
    bgColor += " = \"";
    bgImageStr += " = \"";
    bgCentered += "=";
    bgCentered += cCenteredBg->isChecked() ? "1 # 0/1\n" : "0 # 0/1\n";

    /* Prepare DesktopBackgroundImage string */
    if(lbImages->currentItem() > 0)
    {
        if(cScaleImage->isChecked())
        {
            QWidget *d = QApplication::desktop();
            int sw=d->width();
            int sh=d->height();
            QString imageFile(edDirectory->text());
            QString bgImageFile(icewmCfgDir);;
            edDirectory->setText(bgImageFile);
            bgImageFile += "/IceWMBackgroundImage.img";
            imageFile += lbImages->currentText();
            QImage bgImage(imageFile);
            if(bgImage.isNull())
				QMessageBox::warning(this, tr("Error"), tr("Can't load image"));
            else
            {
                QImage bgScaledImage(bgImage.smoothScale(sw, sh));
                bgScaledImage.save(bgImageFile, QImage::imageFormat(imageFile));
                slotReloadImages();
                QListBoxItem *liCurImage = lbImages->findItem("IceWMBackgroundImage.img");
                lbImages->setCurrentItem(liCurImage);
                bgImageStr += bgImageFile;
            }/* else if(bgImage.isNull()) */
        }/* if(cScaleImage->isChecked()) */
        else
        {
            bgImageStr += edDirectory->text();
            bgImageStr += lbImages->currentText();
        }/* else if(cScaleImage->isChecked()) */
    }/* if(lbImages->currentItem() > 0) */
    bgImageStr += "\"\n";
    bgColor += edColor->text() + "\"\n";



    QString prefFileName(icewmCfgDir + "/preferences");
    //backup copy of preferences
    QString bakPrefFileName(prefFileName);
    bakPrefFileName += ".bak";
    ::rename( QFile::encodeName(prefFileName), QFile::encodeName(bakPrefFileName));

    QFile bakFile(bakPrefFileName);
    QFile prefFile(prefFileName);
    QString s, bgStr;

    if(bakFile.exists())
    {
        if(bakFile.open(IO_ReadOnly) && prefFile.open(IO_WriteOnly))
        {
            QTextStream bakT(&bakFile);
            QTextStream prefT(&prefFile);
            while ((s = bakT.readLine()) != QString::null)
            {
                bgStr = s.stripWhiteSpace();
                if(bgStr.find(DesktopBgColorStr) != -1)
                    prefT << bgColor;
                else
                    if(bgStr.find(DesktopBgImageStr) != -1)
                        prefT << bgImageStr;
                    else
                        if(bgStr.find(DesktopBgCenteredStr) != -1)
                            prefT << bgCentered;
                        else
                            prefT << s << '\n';
            }/* while ((s = bakT.readLine()) != QString::null) */
            bakFile.close();
            prefFile.close();
        }/* if(bakFile.open(IO_ReadOnly)) */
    }/* if(bakFile.exists()) */

    //restart IceWM or icewmbg
	if(cbIceWMVersion->isChecked())
		system("pkill -1 icewm");
	else
		system("pkill -QUIT icewmbg; icewmbg&");
}/* IceBgSetApp::slotSetOptions() */

/**
 * Load current settings of IceWM's desktop
 */
void IceBgSetApp::loadCurrentSettings()
{
    QString prefFileName(icewmCfgDir + "/preferences");

    QFile prefFile(prefFileName);
    QString s, bgStr;
    int     l, r;

    if(prefFile.open(IO_ReadOnly))
        {
            QTextStream prefT(&prefFile);
            while ((s = prefT.readLine()) != QString::null)
            {
                bgStr = s.stripWhiteSpace();
                if(bgStr.find(DesktopBgColorStr) != -1)
                {
                    int g, b;
                    l = bgStr.find('\"');
                    r = bgStr.findRev('\"');
                    bgStr.remove(r, 1000);
                    bgStr.remove(0, l + 1);
                    edColor->setText(bgStr);
                    r = g = b = 0;
                    sscanf(bgStr.data(), bgStr[0] == '#' ? "#%2x%2x%2x" :
                                                           "rgb:%x/%x/%x",
                                                                &r, &g, &b);
                    lPreview->setBackgroundColor(QColor(r, g, b));
                    continue;
                }/* if(bgStr.find(DesktopBgColorStr) != -1) */
                if(bgStr.find(DesktopBgImageStr) != -1)
                {
                    l = bgStr.find('\"');
                    r = bgStr.findRev('\"');
                    bgStr.remove(r, 1000);
                    bgStr.remove(0, l + 1);
                    setPreviewImage(bgStr);
                    r = bgStr.findRev("/");
                    QString imageFile(bgStr.data() + r + 1);
                    bgStr.remove(r + 1, 1000);
                    edDirectory->setText(bgStr);
                    slotReloadImages();

                    QListBoxItem *liCurImage = lbImages->findItem(imageFile);
                    lbImages->setCurrentItem(liCurImage);
                    continue;
                }/* if(bgStr.find(DesktopBgImageStr) != -1) */
                if(bgStr.find(DesktopBgCenteredStr) != -1)
                {
                    l = bgStr.find('=');
                    bgStr.remove(0, l + 1);
                    bgStr.stripWhiteSpace();
                    cCenteredBg->setChecked(atoi(bgStr.data()));
                }/* if(bgStr.find(DesktopBgCenteredStr) != -1) */
            }/* while ((s = bakT.readLine()) != QString::null) */
            prefFile.close();
        }/* if(prefFile.open(IO_ReadOnly)) */
}/* IceBgSetApp::loadCurrentSettings() */

/** No descriptions */
void IceBgSetApp::setPreviewImage(QString &imageFile)
{
    QPixmap previewPixmap(imageFile);
    /* Show preview image */
    if(previewPixmap.isNull())
    {
        lPreview->setText(tr("Can't load image"));
    }/* if(previewImage.isNull()) */
    else
	{
		lPreview->setScaledContents(previewPixmap.width() > lPreview->width() ||
				                    previewPixmap.height() > lPreview->height());
        lPreview->setPixmap(previewPixmap);
    }/* else if(previewImage.isNull()) */

}/*IceBgSetApp::setPreviewImage(QString &imageFile) */
